const AppDisplay = imports.ui.appDisplay;
const St = imports.gi.St;
const Main = imports.ui.main;
const PopupMenu = imports.ui.popupMenu;

const GLib = imports.gi.GLib;
const Gio = imports.gi.Gio;
const Lang = imports.lang;
const Shell = imports.gi.Shell;

let text, button;

const Gettext = imports.gettext;
const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();

const _ = Gettext.domain("mfv-quickoptions").gettext

//--------------------------------------------------------------
//	Tweak Parameters
//--------------------------------------------------------------

// Max items on each folder, -1 for unlimited
const MaxItemsMfv = -1;

// Central folders location
let centralFolder = GLib.get_home_dir() + '/.centralFolder/';

//--------------------------------------------------------------
//	FolderMenuItem class
//--------------------------------------------------------------
const FolderMenuItem = new Lang.Class({
	Name: 'FolderMenuItem',
	Extends: PopupMenu.PopupMenuItem,

	_init: function(name, iconName, iconSize, desktopFile) {
		this.parent(name, null);

		this.item = new PopupMenu.PopupMenuItem(name, { activate: true });

		this.deskFile = desktopFile;
		this.keyfile = new GLib.KeyFile();

		if (this.keyfile.load_from_file(this.deskFile, GLib.KeyFileFlags.NONE) == false)
			return

		this.folderName = name;
		this.appName = null;

		let lang = GLib.get_language_names();
		for (let i=0; i<lang.length; i++) {
            try {this.appName = this.keyfile.get_locale_string('Desktop Entry', 'Name', lang[i]);} catch (e) {}
            if (this.appName != null) break;
        }
        if (this.appName == null) {
            try {this.appName = this.keyfile.get_string('Desktop Entry', 'Name');} catch(e) {}
        }

		//this.appName = this.keyfile.get_string('Desktop Entry', 'Name');

        if (iconName != null) {
            if (iconSize == null) iconSize = 16;
            let icon = null;

            // is '/path/to/filename'?
            if (GLib.path_is_absolute(iconName)) {
                let gicon = null;
                try {
                gicon = new Gio.Icon.new_for_string(iconName);
                } catch (e)  {
                global.logError('' + e);
                }
                if (gicon != null) {
                    icon = new St.Icon({icon_size: iconSize});
                    icon.set_gicon(gicon);
                }
            }
            else {
                // is 'name.png', must be 'name'?
                if (iconName[iconName.length-4] == '.') {
                    let ext = iconName.slice(iconName.length-3);
                    if ( (ext == 'png') || (ext == 'svg') || (ext == 'xpm') )
                        iconName = iconName.slice(0, iconName.length-4);
                }
                // is themed name icon
                icon = new St.Icon({ icon_name: iconName, icon_size: iconSize });
            }
            if (icon)
                this.item.addActor(icon, { align: St.Align.END });
        }

		this.connect('activate', Lang.bind(this, this._onConnect));
	}, 

	// Gets the names of all files that the folder associated with this item has
	_getItemsFromFolder: function() {
		let i = 0;
		let retItems = [];

		if(this.folderName != null) {
			let current = Gio.File.new_for_path(centralFolder + this.folderName);
			let infos = current.enumerate_children('standard::name,standard::type,standard::size', 0, null);

			let FileInf = null;
			while ((FileInf = infos.next_file(null))) {
				retItems[i] = FileInf.get_attribute_as_string('standard::name');
				i++
			}
		}	

		return retItems;
	},

	// Click event handler for the menu item
	_onConnect: function() {
		let tfile = Gio.File.new_for_path(this.deskFile);
		items = this._getItemsFromFolder();

		if(items.indexOf(tfile.get_basename()) != -1) {
			Main.notify(_("Add to Central Launcher"), _("The file you're trying to add is already on the folder."));		
		} else if(items.length < MaxItemsMfv || MaxItemsMfv == -1) {
			let sfile = Gio.File.new_for_path(this.deskFile);
			
			let dfile = Gio.File.new_for_path(centralFolder + this.folderName + "/" + sfile.get_basename());
			
			let callback = null;
			sfile.copy(dfile, 0, null, null, callback);
	
			let notifyString = _("$app added to $folder folder")
			notifyString = notifyString.replace("$app", this.appName);
			notifyString = notifyString.replace("$folder", this.folderName);
			Main.notify(_("Add to Central Launcher"), notifyString);
		} else {
			Main.notify(_("Add to Central Launcher"), _("The selected folder is full. Remove an icon from it first."));
		}
	},

	_getItem: function() {
		return this.item;
	},

	_imageMenuItem: function(name, iconName, iconSize) {
        return 0;
    }
});

//--------------------------------------------------------------
//	QuickOptionsMenuExtension class
//--------------------------------------------------------------
const QuickOptionsMenuExtension = new Lang.Class({
	Name: 'QuickOptionsMenuExtension',

	_init: function() {
		this.shellApp = AppDisplay.AppIconMenu;
		this.origAppIconMenu_redisplay = null;
		this.centralFolder = GLib.get_home_dir() + '/.centralFolder/';
		this.origAppIconMenu_redisplay = null;
		this.folders = this._getMfvFolders();
		this.items = [];

		if (!this.shellApp) return;
	},

	// Scans central folder and returns the names of its child folders
	_getMfvFolders: function() {
		let retFolders = [];
		let current = Gio.File.new_for_path(this.centralFolder);
		let infos = current.enumerate_children('standard::name,standard::type,standard::size', 0, null);
		let i = 0;

		let FileInf = null;
		while ((FileInf = infos.next_file(null))) {
			if (FileInf.get_file_type() == Gio.FileType.DIRECTORY) {
				retFolders[i] = FileInf.get_attribute_as_string('standard::name');
				i += 1;
			}
		}

		return retFolders;
	},

	// Updates the folders (in case one is deleted or renamed)
	_refreshFolders: function() {
		this.folders = this._getMfvFolders();
	},

	_createQuickOptions: function(shellApp, popUpMenu) {
		if (!shellApp || !popUpMenu) return;
   		if (shellApp.is_window_backed()) return;

		let app = shellApp.get_app_info();
		this.desktopFile = app.get_filename();

		if (GLib.file_test(this.desktopFile, GLib.FileTest.EXISTS) == false)
			return;

		this.submenu = new PopupMenu.PopupSubMenuMenuItem(_("Add to Central Launcher"));
		popUpMenu.addMenuItem(this.submenu);

		let n = 0;
		if (this.folders != null && this.submenu != null) {
			for (let i = 0; i < this.folders.length; i++) {
				let item = new FolderMenuItem(this.folders[i], null, null, this.desktopFile);
				this.items[i] = item;
				this.submenu.menu.addMenuItem(this.items[i], n);
				n++;
			}
		}
	}
});

let QuickOptions = new QuickOptionsMenuExtension();
function init() {	
	let localeFolder = ExtensionUtils.getCurrentExtension().dir.get_child('locale');
	Gettext.bindtextdomain("mfv-quickoptions", localeFolder.get_path());
	Gettext.textdomain("mfv-quickoptions");
	global.log(localeFolder.get_path());
}

function enable() {
	QuickOptions.origAppIconMenu_redisplay = AppDisplay.AppIconMenu.prototype._redisplay;
	AppDisplay.AppIconMenu.prototype._redisplay = function() {
	    QuickOptions.origAppIconMenu_redisplay.call(this);
		QuickOptions._refreshFolders();
		QuickOptions._createQuickOptions(this._source.app, this);
	};
}
function disable() {
	AppDisplay.AppIconMenu.prototype._redisplay = QuickOptions.origAppIconMenu_redisplay;
}	
